import os
import threading
from datetime import datetime
from random import random, shuffle, sample, Random, choice

from math import floor, ceil

from pymongo import MongoClient
from scrapy.spiders import CrawlSpider, Rule
from scrapy.linkextractors import LinkExtractor

from scrapy_projects.itzy_bitzy_project.items import URLItem


class BackPointersWebcoinSpider(CrawlSpider):
    name = "backpointerwebcoincrawler"
    mongo_ip = "localhost"  # XXX update at node before running
    mongo_uri = "mongodb://" + mongo_ip
    mongo_db = "webcoin"

    link_extractor = WebcoinLinkExtractor()
    rules = (Rule(link_extractor, callback='parse_item', follow=True, process_links='link_filtering'),)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.crawlerProcess = kwargs['crawlerProcess']
        self.bw = kwargs['bw']
        self.start_urls_tier = kwargs['start_urls_tier']
        self.index_sizes = kwargs['index_sizes']
        self.index_size = choice(self.index_sizes)
        self.current_size_lock = threading.Lock()
        self.num_urls_extracted = 0
        self.collected_urls = []
        self.collected_bodys = []
        self.spider_ip = '.'.join([str(x) for x in os.urandom(5)])

        self.mongo_client = MongoClient(self.mongo_uri)
        self.mongo_db = self.mongo_client[self.mongo_db]

        BackPointersWebcoinSpider.link_extractor.spider = self

    def start_requests(self):
        """
        start_urls_tier:    each tier is created based on the tiers below it.
        seq_in_tier:        0 for the first in the tier, 1 for the 2nd, etc.
        """

        result = list(self.mongo_db.start_urls.find({'start_urls_tier':self.start_urls_tier}))
        start_urls_batches = sorted(result, key=lambda x: x['seq_in_tier'])
        unique_urls = set()
        for start_urls_batch in start_urls_batches:
            tumblr_filtered = [x for x in start_urls_batch['urls'] if 'tumblr' not in x]
            twitter_filtered = [x for x in tumblr_filtered if 'twitter' not in x]
            unique_urls = unique_urls.union(twitter_filtered)
            if len(unique_urls) >= self.index_size*2:
                break
        unique_urls = sorted(list(unique_urls))
        consistent_random = Random()
        with open("scrapy_hash_for_path", "rb") as f:
            seed = f.read()
        consistent_random.seed(seed)
        consistent_random.shuffle(unique_urls)
        unique_urls = unique_urls[:self.index_size*2]
        return [self.make_requests_from_url(x) for x in select_url(unique_urls)]


    def parse_start_url(self, response):
        return self.parse_item(response)

    def parse_item(self, response):
        # drop responses belonging to previous webcoin or path
        # if False:
        with self.current_size_lock:
            if len(self.collected_urls) < self.index_size:
                current_time = datetime.now()
                self.collected_urls.append(response.url)
                self.collected_bodys.append(response.body)

                item = URLItem()
                last_item = len(self.collected_urls) == self.index_size
                if last_item:
                    print('last item found!')

                item['last_item'] = last_item
                item['finish_time'] = current_time
                return item
            else:
                return []


class WebcoinLinkExtractor(LinkExtractor):
    extracted_lock = threading.Lock()
    def extract_links(self, response):
        if len(self.spider.collected_urls) >= self.spider.index_size:
            return []
        else:
            all_extracted_links = LinkExtractor.extract_links(self,response)

            tumblr_filtered = [x for x in all_extracted_links if 'tumblr' not in x.url]
            twitter_filtered = [x for x in tumblr_filtered if 'twitter' not in x.url]
            return select_url(twitter_filtered)


        # with WebcoinLinkExtractor.extracted_lock:
        #     max_to_extract = ceil(self.spider.index_size * 1.5 - self.spider.num_urls_extracted)
        #     if max_to_extract <= 0:
        #         return []
        #     else:
        #         all_extracted_links = LinkExtractor.extract_links(self,response)
        #         if len(all_extracted_links) <= num_links_to_extract:
        #             rand_links = all_extracted_links
        #         else:
        #             rand_links = sample(all_extracted_links, num_links_to_extract)[:max_to_extract]
        #
        #         self.spider.num_urls_extracted += len(rand_links)
        #         return rand_links