import hashlib
import os
import pickle
import random
from datetime import datetime
from time import sleep

from multiprocessing.pool import ThreadPool
import html2text as html2text
import scrapy
from scrapy import signals
from pymongo import MongoClient

mongo_ip = "localhost" #XXX update at node before running
mongo_uri = "mongodb://" + mongo_ip
mongo_db_name = "webcoin"
mongo_client = MongoClient(mongo_uri)
mongo_db = mongo_client[mongo_db_name]


class VerificationSpider(scrapy.Spider):
    name = "urlbreakingspider"
    converter = html2text.HTML2Text()
    converter.ignore_links = True
    pool = ThreadPool(100)
    original_time = datetime(2017, 1, 15, 22, 40, 0)


    def parse(self, response):
        self.pool.map(self.update_mongo, (response,))


    def start_requests(self):
        with open('urls_breaking_list.pk', 'rb') as f:
            urls_list = pickle.load(f)

        urls_list = urls_list

        forever = True
        counter = 0
        urls_per_batch = 1000
        batch_repeats = 4
        while forever:
            counter += 1
            print('counter: %s' % counter)
            urls_batch = random.sample(urls_list, urls_per_batch)
            batch_id = os.urandom(4)
            for attempt_i in range(batch_repeats):  # each one is executed twice to identify wrongly broken ones
                for url in urls_batch:
                    request = self.make_requests_from_url(url)
                    request.meta['batch_id'] = batch_id
                    request.meta['attempt_i'] = attempt_i
                    yield request


    def update_mongo(self, response):
        url = response.url
        text = ''.join(sorted(response.xpath('//body//p//text()').extract()))
        checksum = hashlib.md5(text.encode('utf-8')).hexdigest()
        batch_id = response.meta['batch_id']
        attempt_i = response.meta['attempt_i']
        # print('checksum: %s batch: %s attempt %s' % (checksum, batch_id, attempt_i))
        timestamp = (datetime.now() - self.original_time).total_seconds()
        mongo_dict = {
            'url': url,
            'checksum': checksum,
            'timestamp': timestamp,
            'attempt_i': attempt_i,
            'batch_id': batch_id,
        }
        mongo_db.breaking_urls.insert_one(mongo_dict)